@@ -3,6 +3,7 @@ import scipy.sparse as sps
 from ._numdiff import approx_derivative, group_columns
 from ._hessian_update_strategy import HessianUpdateStrategy
 from scipy.sparse.linalg import LinearOperator
+from scipy._lib._array_api import atleast_nd, array_namespace
 
 
 FD_METHODS = ('2-point', '3-point', 'cs')
@@ -103,8 +104,15 @@ class ScalarFunction:
                              "to be estimated using one of the "
                              "quasi-Newton strategies.")
 
-        # the astype call ensures that self.x is a copy of x0
-        self.x = np.atleast_1d(x0).astype(float)
+        self.xp = xp = array_namespace(x0)
+        _x = atleast_nd(x0, ndim=1, xp=xp)
+        _dtype = xp.float64
+        if xp.isdtype(_x.dtype, "real floating"):
+            _dtype = _x.dtype
+
+        # promotes to floating
+        self.x = xp.astype(_x, _dtype)
+        self.x_dtype = _dtype
         self.n = self.x.size
         self.nfev = 0
         self.ngev = 0
@@ -231,7 +239,9 @@ class ScalarFunction:
                 self.g_prev = self.g
                 # ensure that self.x is a copy of x. Don't store a reference
                 # otherwise the memoization doesn't work properly.
-                self.x = np.atleast_1d(x).astype(float)
+
+                _x = atleast_nd(x, ndim=1, xp=self.xp)
+                self.x = self.xp.astype(_x, self.x_dtype)
                 self.f_updated = False
                 self.g_updated = False
                 self.H_updated = False
@@ -240,7 +250,8 @@ class ScalarFunction:
             def update_x(x):
                 # ensure that self.x is a copy of x. Don't store a reference
                 # otherwise the memoization doesn't work properly.
-                self.x = np.atleast_1d(x).astype(float)
+                _x = atleast_nd(x, ndim=1, xp=self.xp)
+                self.x = self.xp.astype(_x, self.x_dtype)
                 self.f_updated = False
                 self.g_updated = False
                 self.H_updated = False
@@ -308,14 +319,12 @@ class VectorFunction:
                  finite_diff_rel_step, finite_diff_jac_sparsity,
                  finite_diff_bounds, sparse_jacobian):
         if not callable(jac) and jac not in FD_METHODS:
-            raise ValueError("`jac` must be either callable or one of {}."
-                             .format(FD_METHODS))
+            raise ValueError(f"`jac` must be either callable or one of {FD_METHODS}.")
 
         if not (callable(hess) or hess in FD_METHODS
                 or isinstance(hess, HessianUpdateStrategy)):
             raise ValueError("`hess` must be either callable,"
-                             "HessianUpdateStrategy or one of {}."
-                             .format(FD_METHODS))
+                             f"HessianUpdateStrategy or one of {FD_METHODS}.")
 
         if jac in FD_METHODS and hess in FD_METHODS:
             raise ValueError("Whenever the Jacobian is estimated via "
@@ -323,7 +332,16 @@ class VectorFunction:
                              "be estimated using one of the quasi-Newton "
                              "strategies.")
 
-        self.x = np.atleast_1d(x0).astype(float)
+        self.xp = xp = array_namespace(x0)
+        _x = atleast_nd(x0, ndim=1, xp=xp)
+        _dtype = xp.float64
+        if xp.isdtype(_x.dtype, "real floating"):
+            _dtype = _x.dtype
+
+        # promotes to floating
+        self.x = xp.astype(_x, _dtype)
+        self.x_dtype = _dtype
+
         self.n = self.x.size
         self.nfev = 0
         self.njev = 0
@@ -492,14 +510,16 @@ class VectorFunction:
                 self._update_jac()
                 self.x_prev = self.x
                 self.J_prev = self.J
-                self.x = np.atleast_1d(x).astype(float)
+                _x = atleast_nd(x, ndim=1, xp=self.xp)
+                self.x = self.xp.astype(_x, self.x_dtype)
                 self.f_updated = False
                 self.J_updated = False
                 self.H_updated = False
                 self._update_hess()
         else:
             def update_x(x):
-                self.x = np.atleast_1d(x).astype(float)
+                _x = atleast_nd(x, ndim=1, xp=self.xp)
+                self.x = self.xp.astype(_x, self.x_dtype)
                 self.f_updated = False
                 self.J_updated = False
                 self.H_updated = False
@@ -569,7 +589,16 @@ class LinearVectorFunction:
 
         self.m, self.n = self.J.shape
 
-        self.x = np.atleast_1d(x0).astype(float)
+        self.xp = xp = array_namespace(x0)
+        _x = atleast_nd(x0, ndim=1, xp=xp)
+        _dtype = xp.float64
+        if xp.isdtype(_x.dtype, "real floating"):
+            _dtype = _x.dtype
+
+        # promotes to floating
+        self.x = xp.astype(_x, _dtype)
+        self.x_dtype = _dtype
+
         self.f = self.J.dot(self.x)
         self.f_updated = True
 
@@ -578,7 +607,8 @@ class LinearVectorFunction:
 
     def _update_x(self, x):
         if not np.array_equal(x, self.x):
-            self.x = np.atleast_1d(x).astype(float)
+            _x = atleast_nd(x, ndim=1, xp=self.xp)
+            self.x = self.xp.astype(_x, self.x_dtype)
             self.f_updated = False
 
     def fun(self, x):
