@@ -8,12 +8,16 @@ Functions
 """
 import numpy as np
 
-from . import zeros as optzeros
+from . import _zeros_py as optzeros
+from ._numdiff import approx_derivative
 
 __all__ = ['root_scalar']
 
+ROOT_SCALAR_METHODS = ['bisect', 'brentq', 'brenth', 'ridder', 'toms748',
+                       'newton', 'secant', 'halley']
 
-class MemoizeDer(object):
+
+class MemoizeDer:
     """Decorator that caches the value and derivative(s) of function each
     time it is called.
 
@@ -135,6 +139,27 @@ def root_scalar(f, args=(), method=None, bracket=None,
     select one of the derivative-based methods.
     If no method is judged applicable, it will raise an Exception.
 
+    Arguments for each method are as follows (x=required, o=optional).
+
+    +-----------------------------------------------+---+------+---------+----+----+--------+---------+------+------+---------+---------+
+    |                    method                     | f | args | bracket | x0 | x1 | fprime | fprime2 | xtol | rtol | maxiter | options |
+    +===============================================+===+======+=========+====+====+========+=========+======+======+=========+=========+
+    | :ref:`bisect <optimize.root_scalar-bisect>`   | x |  o   |    x    |    |    |        |         |  o   |  o   |    o    |   o     |
+    +-----------------------------------------------+---+------+---------+----+----+--------+---------+------+------+---------+---------+
+    | :ref:`brentq <optimize.root_scalar-brentq>`   | x |  o   |    x    |    |    |        |         |  o   |  o   |    o    |   o     |
+    +-----------------------------------------------+---+------+---------+----+----+--------+---------+------+------+---------+---------+
+    | :ref:`brenth <optimize.root_scalar-brenth>`   | x |  o   |    x    |    |    |        |         |  o   |  o   |    o    |   o     |
+    +-----------------------------------------------+---+------+---------+----+----+--------+---------+------+------+---------+---------+
+    | :ref:`ridder <optimize.root_scalar-ridder>`   | x |  o   |    x    |    |    |        |         |  o   |  o   |    o    |   o     |
+    +-----------------------------------------------+---+------+---------+----+----+--------+---------+------+------+---------+---------+
+    | :ref:`toms748 <optimize.root_scalar-toms748>` | x |  o   |    x    |    |    |        |         |  o   |  o   |    o    |   o     |
+    +-----------------------------------------------+---+------+---------+----+----+--------+---------+------+------+---------+---------+
+    | :ref:`secant <optimize.root_scalar-secant>`   | x |  o   |         | x  | o  |        |         |  o   |  o   |    o    |   o     |
+    +-----------------------------------------------+---+------+---------+----+----+--------+---------+------+------+---------+---------+
+    | :ref:`newton <optimize.root_scalar-newton>`   | x |  o   |         | x  |    |   o    |         |  o   |  o   |    o    |   o     |
+    +-----------------------------------------------+---+------+---------+----+----+--------+---------+------+------+---------+---------+
+    | :ref:`halley <optimize.root_scalar-halley>`   | x |  o   |         | x  |    |   x    |    x    |  o   |  o   |    o    |   o     |
+    +-----------------------------------------------+---+------+---------+----+----+--------+---------+------+------+---------+---------+
 
     Examples
     --------
@@ -154,7 +179,8 @@ def root_scalar(f, args=(), method=None, bracket=None,
     >>> sol.root, sol.iterations, sol.function_calls
     (1.0, 10, 11)
 
-    The `newton` method takes as input a single point and uses the derivative(s)
+    The `newton` method takes as input a single point and uses the
+    derivative(s).
 
     >>> sol = optimize.root_scalar(f, x0=0.2, fprime=fprime, method='newton')
     >>> sol.root, sol.iterations, sol.function_calls
@@ -165,16 +191,20 @@ def root_scalar(f, args=(), method=None, bracket=None,
     >>> def f_p_pp(x):
     ...     return (x**3 - 1), 3*x**2, 6*x
 
-    >>> sol = optimize.root_scalar(f_p_pp, x0=0.2, fprime=True, method='newton')
+    >>> sol = optimize.root_scalar(
+    ...     f_p_pp, x0=0.2, fprime=True, method='newton'
+    ... )
     >>> sol.root, sol.iterations, sol.function_calls
     (1.0, 11, 11)
 
-    >>> sol = optimize.root_scalar(f_p_pp, x0=0.2, fprime=True, fprime2=True, method='halley')
+    >>> sol = optimize.root_scalar(
+    ...     f_p_pp, x0=0.2, fprime=True, fprime2=True, method='halley'
+    ... )
     >>> sol.root, sol.iterations, sol.function_calls
     (1.0, 7, 8)
 
 
-    """
+    """  # noqa: E501
     if not isinstance(args, tuple):
         args = (args,)
 
@@ -224,8 +254,10 @@ def root_scalar(f, args=(), method=None, bracket=None,
                     method = 'halley'
                 else:
                     method = 'newton'
-            else:
+            elif x1 is not None:
                 method = 'secant'
+            else:
+                method = 'newton'
     if not method:
         raise ValueError('Unable to select a solver as neither bracket '
                          'nor starting point provided.')
@@ -235,20 +267,32 @@ def root_scalar(f, args=(), method=None, bracket=None,
 
     try:
         methodc = getattr(optzeros, map2underlying.get(meth, meth))
-    except AttributeError:
-        raise ValueError('Unknown solver %s' % meth)
+    except AttributeError as e:
+        raise ValueError('Unknown solver %s' % meth) from e
 
     if meth in ['bisect', 'ridder', 'brentq', 'brenth', 'toms748']:
         if not isinstance(bracket, (list, tuple, np.ndarray)):
             raise ValueError('Bracket needed for %s' % method)
 
         a, b = bracket[:2]
-        r, sol = methodc(f, a, b, args=args, **kwargs)
+        try:
+            r, sol = methodc(f, a, b, args=args, **kwargs)
+        except ValueError as e:
+            # gh-17622 fixed some bugs in low-level solvers by raising an error
+            # (rather than returning incorrect results) when the callable
+            # returns a NaN. It did so by wrapping the callable rather than
+            # modifying compiled code, so the iteration count is not available.
+            if hasattr(e, "_x"):
+                sol = optzeros.RootResults(root=e._x,
+                                           iterations=np.nan,
+                                           function_calls=e._function_calls,
+                                           flag=str(e), method=method)
+            else:
+                raise
+
     elif meth in ['secant']:
         if x0 is None:
             raise ValueError('x0 must not be None for %s' % method)
-        if x1 is None:
-            raise ValueError('x1 must not be None for %s' % method)
         if 'xtol' in kwargs:
             kwargs['tol'] = kwargs.pop('xtol')
         r, sol = methodc(f, x0, args=args, fprime=None, fprime2=None,
@@ -257,7 +301,15 @@ def root_scalar(f, args=(), method=None, bracket=None,
         if x0 is None:
             raise ValueError('x0 must not be None for %s' % method)
         if not fprime:
-            raise ValueError('fprime must be specified for %s' % method)
+            # approximate fprime with finite differences
+
+            def fprime(x, *args):
+                # `root_scalar` doesn't actually seem to support vectorized
+                # use of `newton`. In that case, `approx_derivative` will
+                # always get scalar input. Nonetheless, it always returns an
+                # array, so we extract the element to produce scalar output.
+                return approx_derivative(f, x, method='2-point', args=args)[0]
+
         if 'xtol' in kwargs:
             kwargs['tol'] = kwargs.pop('xtol')
         r, sol = methodc(f, x0, args=args, fprime=fprime, fprime2=None,
@@ -290,6 +342,9 @@ def _root_scalar_brentq_doc():
     -------
     args : tuple, optional
         Extra arguments passed to the objective function.
+    bracket: A sequence of 2 floats, optional
+        An interval bracketing a root.  `f(x, *args)` must have different
+        signs at the two endpoints.
     xtol : float, optional
         Tolerance (absolute) for termination.
     rtol : float, optional
@@ -309,6 +364,9 @@ def _root_scalar_brenth_doc():
     -------
     args : tuple, optional
         Extra arguments passed to the objective function.
+    bracket: A sequence of 2 floats, optional
+        An interval bracketing a root.  `f(x, *args)` must have different
+        signs at the two endpoints.
     xtol : float, optional
         Tolerance (absolute) for termination.
     rtol : float, optional
@@ -327,6 +385,9 @@ def _root_scalar_toms748_doc():
     -------
     args : tuple, optional
         Extra arguments passed to the objective function.
+    bracket: A sequence of 2 floats, optional
+        An interval bracketing a root.  `f(x, *args)` must have different
+        signs at the two endpoints.
     xtol : float, optional
         Tolerance (absolute) for termination.
     rtol : float, optional
@@ -426,6 +487,9 @@ def _root_scalar_ridder_doc():
     -------
     args : tuple, optional
         Extra arguments passed to the objective function.
+    bracket: A sequence of 2 floats, optional
+        An interval bracketing a root.  `f(x, *args)` must have different
+        signs at the two endpoints.
     xtol : float, optional
         Tolerance (absolute) for termination.
     rtol : float, optional
@@ -445,6 +509,9 @@ def _root_scalar_bisect_doc():
     -------
     args : tuple, optional
         Extra arguments passed to the objective function.
+    bracket: A sequence of 2 floats, optional
+        An interval bracketing a root.  `f(x, *args)` must have different
+        signs at the two endpoints.
     xtol : float, optional
         Tolerance (absolute) for termination.
     rtol : float, optional
