@@ -2,8 +2,7 @@
 # Use the `scipy.optimize` namespace for importing the functions
 # included below.
 
-import warnings
-from . import _nonlin
+from scipy._lib.deprecation import _sub_module_deprecation
 
 
 __all__ = [  # noqa: F822
@@ -53,13 +52,6 @@ def __dir__():
 
 
 def __getattr__(name):
-    if name not in __all__:
-        raise AttributeError(
-            "scipy.optimize.nonlin is deprecated and has no attribute "
-            f"{name}. Try looking in scipy.optimize instead.")
-
-    warnings.warn(f"Please use `{name}` from the `scipy.optimize` namespace, "
-                  "the `scipy.optimize.nonlin` namespace is deprecated.",
-                  category=DeprecationWarning, stacklevel=2)
-
-    return getattr(_nonlin, name)
+    return _sub_module_deprecation(sub_package="optimize", module="nonlin",
+                                   private_modules=["_nonlin"], all=__all__,
+                                   attribute=name)
