@@ -169,7 +169,9 @@ class TestRidgeLines:
         line = _gen_ridge_line([0, 25], test_matr.shape, length, distances, gaps)
         test_matr[line[0], line[1]] = 1
         max_distances = np.full(20, max(distances))
-        identified_lines = _identify_ridge_lines(test_matr, max_distances, max(gaps) + 1)
+        identified_lines = _identify_ridge_lines(test_matr,
+                                                 max_distances,
+                                                 max(gaps) + 1)
         assert_array_equal(identified_lines, [line])
 
     def test_single_bigdist(self):
@@ -182,7 +184,9 @@ class TestRidgeLines:
         max_dist = 3
         max_distances = np.full(20, max_dist)
         #This should get 2 lines, since the distance is too large
-        identified_lines = _identify_ridge_lines(test_matr, max_distances, max(gaps) + 1)
+        identified_lines = _identify_ridge_lines(test_matr,
+                                                 max_distances,
+                                                 max(gaps) + 1)
         assert_(len(identified_lines) == 2)
 
         for iline in identified_lines:
