@@ -3,8 +3,9 @@ Laplacian of a compressed-sparse graph
 """
 
 import numpy as np
-from scipy.sparse import isspmatrix
+from scipy.sparse import issparse
 from scipy.sparse.linalg import LinearOperator
+from scipy.sparse._sputils import convert_pydata_sparse_to_scipy, is_pydata_spmatrix
 
 
 ###############################################################################
@@ -99,7 +100,7 @@ def laplacian(
     Sparse input is reformatted into ``coo`` if ``form="array"``,
     which is the default.
 
-    If the input adjacency matrix is not symmetic, the Laplacian is
+    If the input adjacency matrix is not symmetric, the Laplacian is
     also non-symmetric unless ``symmetrized=True`` is used.
 
     Diagonal entries of the input adjacency matrix are ignored and
@@ -329,6 +330,10 @@ def laplacian(
     in the middle by deleting a single edge.
     Both determined partitions are optimal.
     """
+    is_pydata_sparse = is_pydata_spmatrix(csgraph)
+    if is_pydata_sparse:
+        pydata_sparse_cls = csgraph.__class__
+        csgraph = convert_pydata_sparse_to_scipy(csgraph)
     if csgraph.ndim != 2 or csgraph.shape[0] != csgraph.shape[1]:
         raise ValueError('csgraph must be a square matrix or array')
 
@@ -340,12 +345,12 @@ def laplacian(
 
     if form == "array":
         create_lap = (
-            _laplacian_sparse if isspmatrix(csgraph) else _laplacian_dense
+            _laplacian_sparse if issparse(csgraph) else _laplacian_dense
         )
     else:
         create_lap = (
             _laplacian_sparse_flo
-            if isspmatrix(csgraph)
+            if issparse(csgraph)
             else _laplacian_dense_flo
         )
 
@@ -360,6 +365,8 @@ def laplacian(
         dtype=dtype,
         symmetrized=symmetrized,
     )
+    if is_pydata_sparse:
+        lap = pydata_sparse_cls.from_scipy_sparse(lap)
     if return_diag:
         return lap, d
     return lap
@@ -403,11 +410,11 @@ def _laplacian_sparse_flo(graph, normed, axis, copy, form, dtype, symmetrized):
     if dtype is None:
         dtype = graph.dtype
 
-    graph_sum = graph.sum(axis=axis).getA1()
+    graph_sum = np.asarray(graph.sum(axis=axis)).ravel()
     graph_diagonal = graph.diagonal()
     diag = graph_sum - graph_diagonal
     if symmetrized:
-        graph_sum += graph.sum(axis=1 - axis).getA1()
+        graph_sum += np.asarray(graph.sum(axis=1 - axis)).ravel()
         diag = graph_sum - graph_diagonal - graph_diagonal
 
     if normed:
@@ -456,7 +463,7 @@ def _laplacian_sparse(graph, normed, axis, copy, form, dtype, symmetrized):
     if symmetrized:
         m += m.T.conj()
 
-    w = m.sum(axis=axis).getA1() - m.diagonal()
+    w = np.asarray(m.sum(axis=axis)).ravel() - m.diagonal()
     if normed:
         m = m.tocoo(copy=needs_copy)
         isolated_node_mask = (w == 0)
