@@ -2,8 +2,7 @@
 # Use the `scipy.ndimage` namespace for importing the functions
 # included below.
 
-import warnings
-from . import _measurements
+from scipy._lib.deprecation import _sub_module_deprecation
 
 
 __all__ = [  # noqa: F822
@@ -20,13 +19,6 @@ def __dir__():
 
 
 def __getattr__(name):
-    if name not in __all__:
-        raise AttributeError(
-            "scipy.ndimage.measurements is deprecated and has no attribute "
-            f"{name}. Try looking in scipy.ndimage instead.")
-
-    warnings.warn(f"Please use `{name}` from the `scipy.ndimage` namespace, "
-                  "the `scipy.ndimage.measurements` namespace is deprecated.",
-                  category=DeprecationWarning, stacklevel=2)
-
-    return getattr(_measurements, name)
+    return _sub_module_deprecation(sub_package='ndimage', module='measurements',
+                                   private_modules=['_measurements'], all=__all__,
+                                   attribute=name)
